using System;
using System.Collections.Generic;
using System.Text;
using HAPI;

namespace ConsoleApplication2
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                //construct empty domain 
                Domain domain = new Domain();

                //construct node A 
                LabelledDCNode A = new LabelledDCNode(domain);
                A.SetLabel("A");
                A.SetName("A");
                A.SetNumberOfStates(2);
                for (uint i = 0; i < 2; i++)
                {
                    A.SetStateLabel(i, "state " + i);
                }
                Table tableA = A.GetTable();
                for (uint i = 0; i < tableA.GetSize(); i++)
                {
                    tableA.SetDataItem(i, i + 1);
                }

                //construct node B 
                LabelledDCNode B = new LabelledDCNode(domain);
                B.SetLabel("B");
                B.SetName("B");
                B.SetNumberOfStates(4);
                for (uint i = 0; i < 4; i++)
                {
                    B.SetStateLabel(i, "state " + i);
                }
                //make A parent of B (A->B) 
                B.AddParent(A);
                Table tableB = B.GetTable();
                for (uint i = 0; i < tableB.GetSize(); i++)
                {
                    tableB.SetDataItem(i, i + 1);
                }

                //compile domain 
                domain.Compile();

                //write out probabilities for B 
                Console.WriteLine("No evidence");
                for (uint i = 0; i < B.GetNumberOfStates(); i++)
                {
                    Console.WriteLine("P(B=" + B.GetStateLabel(i) + ")=" + B.GetBelief(i));
                }

                //enter evidence for A and read out probabilities for B 
                Console.WriteLine();
                Console.WriteLine("A=state 0");
                A.SelectState(0);
                domain.Propagate(HAPI.Domain.Equilibrium.H_EQUILIBRIUM_SUM, HAPI.Domain.EvidenceMode.H_EVIDENCE_MODE_NORMAL);
                for (uint i = 0; i < B.GetNumberOfStates(); i++)
                {
                    Console.WriteLine("P(B=" + B.GetStateLabel(i) + "|e)=" + B.GetBelief(i));
                }

                //enter some other evidece for A and read out probabilities for B 
                Console.WriteLine();
                Console.WriteLine("A=state 1");
                A.SelectState(1);
                domain.Propagate(HAPI.Domain.Equilibrium.H_EQUILIBRIUM_SUM, HAPI.Domain.EvidenceMode.H_EVIDENCE_MODE_NORMAL);
                for (uint i = 0; i < B.GetNumberOfStates(); i++)
                {
                    Console.WriteLine("P(B=" + B.GetStateLabel(i) + "|e)=" + B.GetBelief(i));
                }

                //clean up (release all reasources allocated by domain) 
                domain.Delete();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
            Console.ReadLine();
        }
    }
}
